/*
 * Mkiturvat v0.031 by Heikki Kernen
 * Additional ideas and rigorous beta testing by Eero Vyrynen
 *
 * Based on the test level by Jetro Lauha
 *
 * Changes in 0.031
 *	- Tweaked scores needed for various ranks (too big a gap from 150k to 200k)
 *	- Tweaked landing bonuses
 * Changes in 0.03
 *	- English version included
 *	- Added ranks
 *	- Added tower and stand in the background to help maneuvering while high up in the air
 *	- Added bonuses for various types of landings (Head first, feet first etc.)
 *	- Body parts flash red when hit
 * Changes in 0.02
 *	- First public release
 */

uint mTime;
int screenWidth, screenHeight;

int score = 0, headHits = 0, headLanding = 0, headBonus = 0, distanceBonus = 0, feetFirst = 0, neckInjury = 0, elbowDamage = 0, mahalasku = 0;
float distance, finalDistance;
float hyppyriX;

GameApp *app;
UIFont *medFont, *smallFont;
uint medFontHeight, smallFontHeight;
PhysicsWorld *pw;

PhysicsObject *head, *neck, *trunk, *rthigh, *lthigh, *rcalf, *lcalf, *rfore, *lfore, *larm, *rarm, *rfoot, *lfoot;

PhysicsObject *objHyppyri;

bool finished = false;
bool aborted = false;

bool firstTouch = false;
bool secondTouch = false;

bstr first = "";
bstr second = "";

// Called once on level initialization.
void onInit()
{
	app = getGameApp();
	screenWidth = app->getScreenWidth();
	screenHeight = app->getScreenHeight();
	medFont = app->getFont("medium");
	smallFont = app->getFont("small");
	medFontHeight = medFont->getFontHeight();
	smallFontHeight = smallFont->getFontHeight();

	app->setClearColor(0, 0.7, 1.0);

	pw = app->getPhysicsWorld();
	pw->setGravity(0, 0, -0.5);

	head = pw->findObject("Head");
	neck = pw->findObject("Neck");
	trunk = pw->findObject("Trunk");
	rthigh = pw->findObject("RightThigh");
	lthigh = pw->findObject("LeftThigh");
	rarm = pw->findObject("RightArm");
	larm = pw->findObject("LeftArm");
	rfore = pw->findObject("RightForearm");
	lfore = pw->findObject("LeftForearm");
	rcalf = pw->findObject("RightCalf");
	lcalf = pw->findObject("LeftCalf");
	rfoot = pw->findObject("RightFoot");
	lfoot = pw->findObject("LeftFoot");

	revertColors();

	float x, y, z = 0;
	objHyppyri = pw->findObject("Hyppyri");
	objHyppyri->getInitialPosition(x, y, z);
	hyppyriX = x;

	// no apparent way to get the size of the box, so hard-coded
	hyppyriX += 6.456250 / 2;
}

void revertColors() {
	head->setColor(1,0.7,0.5);
	neck->setColor(0,0.75,1);
	trunk->setColor(0,0.75,1);
	rthigh->setColor(0,0.75,1);
	lthigh->setColor(0,0.75,1);
	rarm->setColor(0,0.75,1);
	larm->setColor(0,0.75,1);
	rfore->setColor(1,0.7,0.5);
	lfore->setColor(1,0.7,0.5);
	rcalf->setColor(0,0.75,1);
	lcalf->setColor(0,0.75,1);
	rfoot->setColor(0.2,1,0.75);
	lfoot->setColor(0.2,1,0.75);
}


// Called once for each game logic update before the physics update.
// Parameter 'time' is the simulation time in milliseconds.
void onUpdate(uint time)
{
	mTime = time;

	float x, y, z = 0;
	head->getPosition(x, y, z);
	if (!aborted && z < -120 && !finished)
	{
		aborted = true;
		setGameState(GAMESTATE_ABORTED);
	}

	if (x > 2.5) pw->setGravity(2.5, 0, -12.5);
	if (x > hyppyriX) pw->setGravity(0, 0, -2.0);
	distance = (x-hyppyriX);
	if (distance < 0) distance = 0;

	revertColors();
}


// Called when there is a collision between the two given objects.
// Returns true to create contacts or false to ignore collision.
bool onCollision(PhysicsObject *o1, PhysicsObject *o2)	{
	bstr o1name = o1->getName();
	bstr o2name = o2->getName();

	if (o1->getType() == PHYSOBJ_RAGDOLL_BODY) o1->setColor(1,0,0);
	if (o2->getType() == PHYSOBJ_RAGDOLL_BODY) o2->setColor(1,0,0);

	if (o1name == "Head" || o2name == "Head") {
		headHits++;
	}

	if ((o1name == "lask" && o2name != "lask") || (o2name == "lask" && o1name != "lask")  && !finished && !firstTouch) {
		if (o1name != "lask") first = o1name;
		else first = o2name;
		firstTouch = true;
		return true;
	}

	if ((o1name == "lask" && o2name != "lask") || (o2name == "lask" && o1name != "lask")  && !finished && firstTouch) {
	  
		if (o1name != "lask" && o1name != first) {
			second = o1name;
			secondTouch = true;
		} else if (o2name != "lask" && o2name != first) {
			second = o2name;
			secondTouch = true;
		}

		if (secondTouch) {
		        finished = true;
			// setGameState(GAMESTATE_FINISHED);
			distanceBonus = int(distance * 1000);
			headBonus = headHits * 100;

			if (headBonus > 5000) headBonus = 5000;

			// style bonus if you fall on your head
			if (first == "Head") headLanding = 5000;
			if ((first == "RightFoot" && second == "LeftFoot") || (first == "LeftFoot" && second == "RightFoot")) feetFirst = 7500;
			if (first == "Neck") neckInjury = 10000;
			if ((first == "RightForearm" && second == "LeftForearm") || (first == "LeftForearm" && second == "RightForearm")) elbowDamage = 5000;
			if (first == "Trunk") mahalasku = -5000;

			finalDistance = distance;
			score += feetFirst;
			score += neckInjury;
			score += elbowDamage;
			score += mahalasku;
			score += headLanding;
			score += headBonus;
			score += distanceBonus;
		}
	} 
	// generate collision
	return true;
}


// Called when the game view is rendered.
// Might happen more or less frequently than update calls.
void onRender()	{
	if (mTime < 5000) {
		medFont->drawString(float(screenWidth / 2), 70, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_BOTTOM, "Mkiturvat v0.031");
		smallFont->drawString(float(screenWidth / 2), 90, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_BOTTOM, "\"Who needs skis, anyway?\"");
    	}
	if (!finished) {
		smallFont->drawString(100,20, UIFONT_ALIGN_HORIZ_CENTER, "Distance: " + bstrFormat(distance) + "m");
	} else {
		smallFont->drawString(100,20, UIFONT_ALIGN_HORIZ_CENTER, "Distance: " + bstrFormat(finalDistance) + "m");
	}
	if (aborted)
		drawFailure();
	if (finished)
	drawVictory();
}

void drawFailure()
{
	medFont->drawString(float(screenWidth / 2), float(screenHeight / 2), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Matti fell to his death!");
}


void drawVictory()
{
	float r=0, g=0, b=0, a=1;

	medFont->drawString(float(screenWidth / 2), float(screenHeight / 4), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Landing successful!");
	medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * 2), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Distance: " + bstrFormat(distanceBonus));
	medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * 4), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Brain damage: " + bstrFormat(headBonus));

	int x=6;

	if (headLanding != 0) {
		medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * 6), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Landed on head: +" + bstrFormat(headLanding),0,1,0,1);
		x+=2;
	} else if (neckInjury != 0) {
		medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * 6), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Broken neck: +" + bstrFormat(neckInjury),0,1,0,1);
		x+=2;
	} else if (feetFirst != 0) {
		medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * 6), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER,"Feet first: +" + bstrFormat(feetFirst),0,1,0,1);
		x+=2;
	} else if (elbowDamage != 0) {
		medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * 6), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Crushed elbows: +" + bstrFormat(elbowDamage),0,1,0,1);
		x+=2;
	} else if (mahalasku != 0) {
		medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * 6), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Fell flat on stomach: " + bstrFormat(mahalasku),1,0,0,1);
		x+=2;
	}

	medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * x), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Total score: " + bstrFormat(score));

	bstr title = "Total failure (10th)";

	if (score > 0) {
		title = "Newbie (9th)";
		r = 1;
		g = 0;
		b = 0;
	}
	if (score > 29999) {
		title = "Beginner (8th)";
		r = 1;
		g = 0.33;
		b = 0;
	}
	if (score > 69999) {
		title = "Intermediate (7th)";
		r = 1;
		g = 0.66;
		b = 0;
	}

	if (score > 109999) {
		title = "Advanced (6th)";
		r = 1;
		g = 1;
		b = 0;
	}
	if (score > 149999) {
		title = "Expert (5th)";
		r = 0.66;
		g = 1;
		b = 0;
	}
	if (score > 174999) {
		title = "Professional (4th)";
		r = 0;
		g = 1;
		b = 0;
	}
	if (score > 199999) {
		title = "Master (3rd)";
		r = 0;
		g = 0;
		b = 1;
	}
	if (score > 224999) {
		title = "Legend (2nd)";
		r = 1;
		g = 0;
		b = 1;
	}

	// BETA TESTER RECORD == 269m

	if (score > 249999) {
		title = "Matti himself? (1st)";
		r = 0;
		g = 0;
		b = 0;
	}

	medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * (x+4)), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Rank:");
	medFont->drawString(float(screenWidth / 2), float(screenHeight / 4 + int(medFontHeight) * (x+5)), UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, title, r, g, b, a);
}
